'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createModule;

var _nuclearJs = require('nuclear-js');

var _lodash = require('lodash.foreach');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.mapvalues');

var _lodash4 = _interopRequireDefault(_lodash3);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createModule(name) {
  var _ref = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var _ref$stores = _ref.stores;
  var stores = _ref$stores === undefined ? {} : _ref$stores;
  var _ref$actions = _ref.actions;
  var actions = _ref$actions === undefined ? {} : _ref$actions;
  var _ref$getters = _ref.getters;
  var getters = _ref$getters === undefined ? {} : _ref$getters;
  var StoreFactory = _ref.StoreFactory;

  (0, _invariant2.default)('string' === typeof name, 'createModule(): You must provide a valid name as first argument.');

  var Factory = function Factory(reactor) {
    StoreFactory = StoreFactory || _nuclearJs.Store;

    var evaluate = reactor.evaluate.bind(reactor);
    var dispatch = reactor.dispatch.bind(reactor);
    var observe = reactor.observe.bind(reactor);
    var registerStores = reactor.registerStores.bind(reactor);

    var realStores = createStores(stores, StoreFactory, evaluate);
    var realGetters = createGetters(getters, { evaluate: evaluate });
    var realActions = createActions(actions, { dispatch: dispatch, evaluate: evaluate });
    var realObservers = createObservers(getters, { observe: observe });

    registerStores(realStores);

    return {
      name: name,
      getters: realGetters,
      actions: realActions,
      observers: realObservers
    };
  };

  Factory.getters = getters;
  Factory.actions = actions;

  return Factory;
}

function createStores(stores, StoreFactory, evaluate) {
  var realStores = {};

  (0, _lodash2.default)(stores, function (store, name) {
    if (evaluate([name])) {
      return;
    }
    realStores[name] = StoreFactory(store);
  });

  return realStores;
}

// fooModule.getters.foo()
// => will return evaluated value
function createGetters(getters, _ref2) {
  var evaluate = _ref2.evaluate;

  return (0, _lodash4.default)(getters, function (getter) {
    return function () {
      var transform = arguments.length <= 0 || arguments[0] === undefined ? identity : arguments[0];
      return transform(evaluate(getter));
    };
  });
}

// fooModule.observers.foo(foo => console.log('foo changed %s', foo))
function createObservers(getters, _ref3) {
  var observe = _ref3.observe;

  return (0, _lodash4.default)(getters, function (getter) {
    return function (handler) {
      return observe(getter, handler);
    };
  });
}

function createActions(actions, _ref4) {
  var dispatch = _ref4.dispatch;
  var evaluate = _ref4.evaluate;

  return (0, _lodash4.default)(actions, function (action) {
    return action({ dispatch: dispatch, evaluate: evaluate });
  });
}

var identity = function identity(x) {
  return x;
};